#include "global.h"
#include "music_player.h"

FONT *titlefont = NULL;
FONT *focusfont = NULL;
FONT *normalfont = NULL;

BITMAP *floor_tile = NULL;
BITMAP *back_tile = NULL;
BITMAP *tank_sprite[4];
BITMAP *rocket_sprite;
BITMAP *drone_sprite[8];
BITMAP *player_sprite[2];

SAMPLE *cling_sample;
SAMPLE *shot_sample;
SAMPLE *explode_sample;

SCREENSHOT *screenshot;
CFPS fps;
HSC *hsc = 0;

GlobalWidgets *globalWidgets = 0;

int soundVolume = 7;
int musicVolume = 7;

float SIN[LUT_SIZE+1];
float COS[LUT_SIZE+1];

void TextButton::Draw(Bitmap &canvas) {
	FONT *f = HasFocus() ? focusfont : normalfont;
	int offset = Selected() ? 2 : 0;

	switch (GetAlignment()) {
		case 0:
			textprintf_ex(canvas, f, 4+offset, -8+offset, Color::black, -1, GetText());
			textprintf_ex(canvas, f, offset, -10+offset, -1, -1, GetText());
		break;

		case 1:
			textprintf_right_ex(canvas, f, w()+offset, -8+offset, Color::black, -1, GetText());
			textprintf_right_ex(canvas, f, w()-4+offset, -10+offset, -1, -1, GetText());
		break;

		case 2:
			textprintf_centre_ex(canvas, f, w()/2+4+offset, -8+offset, Color::black, -1, GetText());
			textprintf_centre_ex(canvas, f, w()/2+offset, -10+offset, -1, -1, GetText());
		break;
	};
}


ChoiceButton::ChoiceButton() : TextButton() {
	selection = -1;
}


char *ChoiceButton::GetText() {
	if (selection >= 0 && selection < choice.size()) {
		return (char *)choice[selection].c_str();
	}

	return "";
}


void ChoiceButton::Increment() {
	if (choice.size() == 2 || selection < choice.size()-1) {
		++selection;
		selection %= choice.size();
	}
}


void ChoiceButton::Decrement() {
	if (choice.size() == 2 || selection > 0) {
		--selection;
		if (selection < 0) {
			selection += choice.size();
		}
	}
}


bool ChoiceButton::MsgChar(int c) {
	if (TextButton::MsgChar(c)) {
		return true;
	}

	int old_sel = selection;

	switch (c>>8) {
		case KEY_LEFT:		Decrement();	break;
		case KEY_RIGHT:		Increment();	break;
		default:			return false;
	};

	if (selection != old_sel) {
		GetParent()->HandleEvent(*this, MSG_ACTIVATE, selection, old_sel);
		PlaySample(Skin::SAMPLE_ACTIVATE);
	}

	return true;
}

void ChoiceButton::MsgLPress() {
	Widget::MsgLPress();

	int old_sel = selection;
	Increment();
	if (selection != old_sel) {
		GetParent()->HandleEvent(*this, MSG_ACTIVATE, selection, old_sel);
		PlaySample(Skin::SAMPLE_ACTIVATE);
	}
}

void ChoiceButton::MsgRPress() {
	Widget::MsgRPress();
	int old_sel = selection;
	Decrement();
	if (selection != old_sel) {
		GetParent()->HandleEvent(*this, MSG_ACTIVATE, selection, old_sel);
		PlaySample(Skin::SAMPLE_ACTIVATE);
	}
}


ScrollingText::ScrollingText() : Label(), tick(0) {
}


void ScrollingText::Draw(Bitmap &canvas) {
	canvas.Clear();
	Font f = GetFont(0);
	const char *t = GetText();
	int l = f.TextLength(t);
	int dx = (tick)%l;
	f.TextPrintf(canvas, Point(-dx, 0), Color::white, -1, 0, t);
	//textprintf_ex(canvas, f, -dx, 0, -1, -1, 0, t);

	if (-dx + l < w()) {
		f.TextPrintf(canvas, Point(-dx + l, 0), Color::white, -1, 0, t);
	}
}


void ScrollingText::MsgTick() {
	Label::MsgTick();
	++tick;
}


void ScrollingText::LoadText(const char *filename) {
	PACKFILE *f = pack_fopen(filename, "r");
	char t[4096];
	pack_fgets(t, 4096, f);
	Label::SetText(t);
	pack_fclose(f);
}


void CustomLabel::Draw(Bitmap &canvas) {
	canvas.Clear(Color(80,80,80));

	Color col = Color(0,192,0);
	textprintf_centre_ex(canvas, titlefont, w()/2+2, -16, col, -1, GetText());
	textprintf_centre_ex(canvas, titlefont, w()/2-2, -16, col, -1, GetText());
	textprintf_centre_ex(canvas, titlefont, w()/2, -14, col, -1, GetText());
	textprintf_centre_ex(canvas, titlefont, w()/2, -18, col, -1, GetText());
	//textprintf_centre_ex(canvas, titlefont, w()/2+4, -18, Color::black, -1, GetText());
	textprintf_centre_ex(canvas, titlefont, w()/2, -16, -1, -1, GetText());

	hline(canvas, 0, 0, w()-1, col);
	hline(canvas, 0, h()-1, w()-1, col);

	col = Color(0,128,0);
	hline(canvas, 0, 1, w()-1, col);
	hline(canvas, 0, h()-2, w()-1, col);
}


void Gradient(Bitmap &canvas, int x1, int y1, int x2, int y2, Color c1, Color c2) {
	float h2 = (float)(y2 - y1);

	float r1 = (float)c1.r();
	float g1 = (float)c1.g();
	float b1 = (float)c1.b();

	float r2 = (float)c2.r();
	float g2 = (float)c2.g();
	float b2 = (float)c2.b();

	float dr = (r2 - r1)/h2;
	float dg = (g2 - g1)/h2;
	float db = (b2 - b1)/h2;

	for (int i=y1; i<y2; i++) {
		hline(canvas, x1, i, x2, makecol((int)(r1+=dr), (int)(g1+=dg), (int)(b1+=db)));
	}
}


GameDialog::GameDialog() : Dialog() {
	acc.Add(KEY_F12, 0, 1000);
	acc.Add(KEY_PRTSCR, 0, 1000);
	Add(acc);
}


void GameDialog::HandleEvent(Widget &obj, int msg, int arg1, int arg2) {
	Dialog::HandleEvent(obj, msg, arg1, arg2);

	if (msg == 1000) {
		take_screenshot(screenshot, GetDriver()->GetCanvas());
	}
}


void GameDialog::MsgTick() {
	Dialog::MsgTick();
	fps.Tick();
	poll_music();
}


void GameDialog::MsgDraw() {
	Dialog::MsgDraw();
	fps.Frame();

	if (MAS::Settings::showFrameRate) {
		Bitmap canvas = Bitmap(GetDriver()->GetCanvas());
		skin->GetFont(1).TextPrintf(canvas, Point(0, 0), Color::green, Color::black, 0, "%d FPS", fps.Get());
	}
}



void Background::Draw(Bitmap &canvas) {
	Gradient(canvas, 0, 0, w()-1, h(), Color(96,96,96), Color(128,128,128));
	theSkin->GetFont(1).TextPrintf(canvas, SCREEN_W, SCREEN_H-15, Color(0,0,0), -1, 1, "Dodger v%d.%02d", DODGER_VERSION_MAJOR, DODGER_VERSION_MINOR);
	theSkin->GetFont(1).TextPrintf(canvas, SCREEN_W-1, SCREEN_H-16, Color(192,192,192), -1, 1, "Dodger v%d.%02d", DODGER_VERSION_MAJOR, DODGER_VERSION_MINOR);
}


void GameDialog::AnimWidget::MsgStart() {
	ClearScreen::MsgStart();
	startedAnimating = false;
}


void GameDialog::AnimWidget::Draw(Bitmap &canvas) {
	if (!startedAnimating) {
		anim.Reset();
		//Bitmap canvas = Bitmap(GetParent()->GetDriver()->GetCanvas());
		anim.Create(canvas, 200);
		startedAnimating = true;
	}

	if (anim.Animating()) {
		anim.Draw(canvas, Animator::CLOSE_HORIZONTAL);
	}
}


bool GameDialog::AnimWidget::MsgWantmouse() {
	return false;
}


void GameDialog::AnimWidget::MsgDraw() {

	ClearScreen::MsgDraw();
}


void GameDialog::AnimWidget::MsgTick() {
	ClearScreen::MsgTick();
	if (anim.Animating()) {
		anim.Update();
	}
}


void GameDialog::AnimWidget::Transition::Create(Bitmap &bmp, int length) {
	Animator::Create(bmp, length);
	back.Clear(Color::black);
}

Cursor *cur = 0;

void LoadData() {
	char full[1024];

DEBUG
	PALETTE pal;
	MakeFullPath(full, "data/fontbig.pcx");	titlefont = load_font(full, pal, 0);
	MakeFullPath(full, "data/fontfx.pcx");	focusfont = load_font(full, pal, 0);
	MakeFullPath(full, "data/fontfx2.pcx");	normalfont = load_font(full, pal, 0);
	set_palette(pal);

DEBUG
	//TODO: load bitmaps, sounds, music, etc.
	MakeFullPath(full, "data/floor.tga");	floor_tile = load_bitmap(full, 0);
	MakeFullPath(full, "data/back.tga");	back_tile = load_bitmap(full, 0);
	MakeFullPath(full, "data/tank1.tga");	tank_sprite[0] = load_bitmap(full, 0);
	MakeFullPath(full, "data/tank2.tga");	tank_sprite[1] = load_bitmap(full, 0);
	MakeFullPath(full, "data/tank3.tga");	tank_sprite[2] = load_bitmap(full, 0);
	MakeFullPath(full, "data/tank4.tga");	tank_sprite[3] = load_bitmap(full, 0);
	MakeFullPath(full, "data/rocket.tga");	rocket_sprite = load_bitmap(full, 0);
	MakeFullPath(full, "data/drone1a.tga");	drone_sprite[0] = load_bitmap(full, 0);
	MakeFullPath(full, "data/drone1b.tga");	drone_sprite[1] = load_bitmap(full, 0);
	MakeFullPath(full, "data/drone2a.tga");	drone_sprite[2] = load_bitmap(full, 0);
	MakeFullPath(full, "data/drone2b.tga");	drone_sprite[3] = load_bitmap(full, 0);
	MakeFullPath(full, "data/drone3a.tga");	drone_sprite[4] = load_bitmap(full, 0);
	MakeFullPath(full, "data/drone3b.tga");	drone_sprite[5] = load_bitmap(full, 0);
	MakeFullPath(full, "data/drone4a.tga");	drone_sprite[6] = load_bitmap(full, 0);
	MakeFullPath(full, "data/drone4b.tga");	drone_sprite[7] = load_bitmap(full, 0);
	MakeFullPath(full, "data/player1.tga");	player_sprite[0] = load_bitmap(full, 0);
	MakeFullPath(full, "data/player2.tga");	player_sprite[1] = load_bitmap(full, 0);

	MakeFullPath(full, "data/cling.wav");	cling_sample = load_sample(full);
	MakeFullPath(full, "data/shot.wav");	shot_sample = load_sample(full);
	MakeFullPath(full, "data/explode.wav");	explode_sample = load_sample(full);

DEBUG
/*
	cur = new Cursor;
	MakeFullPath(full, "data/cursor.tga");
	BITMAP *tmpCur = load_bitmap(full, NULL);
	cur->Create(tmpCur, 6);
	cur->SetFocus(0, 0);
	cur->SetAnimationInterval(100);
	theSkin->SetCursor(*cur, Skin::MOUSE_NORMAL);
	destroy_bitmap(tmpCur);
*/

	// make a title
	globalWidgets->title.Shape(0, 6, 100, 14, 1);
	globalWidgets->title.ClearFlag(D_AUTOSIZE);
	globalWidgets->title.AlignCentre();
	globalWidgets->title.SetText("D o d g e r");

	// text scroller
	globalWidgets->lblBottom.Shape(0, SCREEN_H-24, SCREEN_W, 24);
	MakeFullPath(full, "data/credits");
	globalWidgets->lblBottom.LoadText(full);

	MakeFullPath(full, "data/0053.it");
	music = dumb_load_it(full);

DEBUG
}

void UnloadData() {
DEBUG
	destroy_font(titlefont);
	destroy_font(focusfont);
	destroy_font(normalfont);

DEBUG
	// TODO: destroy extra data
	destroy_bitmap(floor_tile);
DEBUG
	destroy_bitmap(back_tile);
DEBUG
	destroy_bitmap(tank_sprite[0]);
	destroy_bitmap(tank_sprite[1]);
	destroy_bitmap(tank_sprite[2]);
	destroy_bitmap(tank_sprite[3]);
DEBUG
	destroy_bitmap(rocket_sprite);
DEBUG
	destroy_bitmap(drone_sprite[0]);
	destroy_bitmap(drone_sprite[1]);
	destroy_bitmap(drone_sprite[2]);
	destroy_bitmap(drone_sprite[3]);
	destroy_bitmap(drone_sprite[4]);
	destroy_bitmap(drone_sprite[5]);
	destroy_bitmap(drone_sprite[6]);
	destroy_bitmap(drone_sprite[7]);

	destroy_bitmap(player_sprite[0]);
	destroy_bitmap(player_sprite[1]);

	destroy_sample(cling_sample);
	destroy_sample(shot_sample);
	destroy_sample(explode_sample);

	destroy_screenshot(screenshot);
}
